@@ -4,16 +4,21 @@
  * usage: setvesablank ON|on|off
  */
 #include <stdio.h>
-#include <linux/termios.h>
-
-extern int getfd();
+#include <sys/ioctl.h>
+#include "getfd.h"
+#include "nls.h"
 
+int
 main(int argc, char *argv[]) {
     int fd;
     struct { char ten, onoff; } arg;
 
+    setlocale(LC_ALL, "");
+    bindtextdomain(PACKAGE, LOCALEDIR);
+    textdomain(PACKAGE);
+
     if (argc != 2) {
-       fprintf(stderr, "usage: setvesablank ON|on|off\n");
+       fprintf(stderr, _("usage: %s\n"), "setvesablank ON|on|off");
        exit(1);
     }
     fd = getfd();
@@ -27,5 +32,5 @@ main(int argc, char *argv[]) {
        perror("setvesablank: TIOCLINUX");
        exit(1);
     }
-    exit(0);
+    return 0;
 }
